/* roms.rc: ROM selector dialog box
   Copyright (c) 2008 Marek Januszewski

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

   Author contact information:

   E-mail: philip-fuse@shadowmagic.org.uk

*/

#include "roms.h"

IDD_ROMS DIALOGEX 0,0,163,84
STYLE DS_3DLOOK | DS_CENTER | DS_MODALFRAME | DS_FIXEDSYS | WS_VISIBLE | WS_BORDER | WS_CAPTION | WS_DLGFRAME | WS_POPUP | WS_SYSMENU
EXSTYLE WS_EX_APPWINDOW
CAPTION "Fuse - Select ROMs"
FONT 8,"Ms Shell Dlg 2",400,0,1
BEGIN
/* this is created at runtime in roms.c 
   using the positions and sizes like below */
/*   
    GROUPBOX        "ROM 0",IDC_STATIC,0,0,160,30
    EDITTEXT        IDC_EDIT1,5,10,110,16,ES_AUTOHSCROLL
    PUSHBUTTON      "Select...",IDC_BUTTON1,120,10,35,14

    GROUPBOX        "ROM 1",IDC_STATIC,0,30,160,30
    EDITTEXT        IDC_EDIT2,5,40,110,16,ES_AUTOHSCROLL
    PUSHBUTTON      "Select...",IDC_BUTTON2,120,40,35,14
*/

    DEFPUSHBUTTON   "OK",IDOK,25,70,50,14
    PUSHBUTTON      "Cancel",IDCANCEL,85,65,50,14
END
